mod websocket;
mod events;

use iced::widget::{column, Column, button, text_input, text};
use iced::{Application, Command, Element, Pixels, settings::Settings};
use iced_futures::Subscription;
use websocket::{websocket, AppWebsocketConfig};
use events::ChatAppEvent;

fn main() -> iced::Result {
    let settings = Settings {
        id: None,
        window: Default::default(),
        flags: (),
        fonts: vec![],
        default_font: Default::default(),
        default_text_size: Pixels(16.0),
        antialiasing: true,
    };

    ChatApp::run(settings)
}

#[derive(Default)]
struct ChatApp {
    host: String
}

impl ChatApp {
    fn view_auth(&self) -> Element<<ChatApp as Application>::Message> {
        column![
            text_input("Host", &self.host)
                .on_input(ChatAppEvent::HostTextInputChanged)
                .on_submit(ChatAppEvent::ConnectButtonPress),
            button("Connect")
                .on_press(ChatAppEvent::ConnectButtonPress)
        ].into()
    }
}

impl Application for ChatApp {
    type Executor = iced_futures::backend::native::tokio::Executor;
    type Message = ChatAppEvent;
    type Theme = iced::theme::Theme;
    type Flags = ();

    fn new(flags: Self::Flags) -> (Self, Command<Self::Message>) {
        (Self {
            host: String::from("127.0.0.1:3000"),
        }, Command::none())
    }

    fn title(&self) -> String {
        String::from("Chat app")
    }

    fn update(&mut self, message: ChatAppEvent) -> Command<Self::Message> {
        println!("update");
        match message {
            ChatAppEvent::ConnectButtonPress => {
                println!("ConnectButtonPress")
            },
            ChatAppEvent::HostTextInputChanged(host_value) => {
                self.host = host_value;
            }
        }

        Command::none()
    }

    fn view(&self) -> Element<Self::Message> {
        println!("view");
        self.view_auth()
    }

    fn subscription(&self) -> Subscription<Self::Message> {
        println!("subscription");
        websocket(AppWebsocketConfig::new(self.host.clone(), None, None))
    }
}